import base64
import json
from enola.base.common.huemul_connection import HuemulConnection
from enola.base.common.huemul_functions import HuemulFunctions
from enola.base.common.huemul_response_error import HuemulResponseError
from enola.base.common.huemul_response_to_bloc import HuemulResponseToBloc
from enola.enola_types import ExecutionQueryModel, ExecutionResponseModel

class EnolaExecutionProvider(HuemulResponseToBloc):
    
    #
    # execution_get
    # @param ExecutionModel executionModel
    # @return AgentExecuteResponseModel[AgentExecuteResponseModel]
    #
    def execution_get(self, execution_query_model: ExecutionQueryModel):
        #self = AgentExecuteResponseModel()
        try:
            hf = HuemulFunctions()
            excludeFilter = []

            queryParams= [
                    {"name": "page", "value": execution_query_model.page_number},
                    {"name": "limit", "value": execution_query_model.limit},
                    {"name": "agentExecStartDTFrom", "value": execution_query_model.date_from},
                    {"name": "agentExecStartDTTo", "value": execution_query_model.date_to},
                    {"name": "includeTags", "value": execution_query_model.includeTags},
                    {"name": "includeData", "value": execution_query_model.includeData},
                    {"name": "includeErrors", "value": execution_query_model.includeErrors},
                    {"name": "includeEvals", "value": execution_query_model.includeEvals},
                    {"name": "agentExecType", "value": "START"}
                ]

            if (execution_query_model.chamber_id_list != None and len(execution_query_model.chamber_id_list) > 0):
                queryParams.append({"name": "chamberId", "value": json.dumps(execution_query_model.chamber_id_list)})

            if (execution_query_model.agent_id_list != None and len(execution_query_model.agent_id_list) > 0):
                queryParams.append({"name": "agentId", "value": json.dumps(execution_query_model.agent_id_list)})

            if (execution_query_model.agent_deploy_id_list != None and len(execution_query_model.agent_deploy_id_list) > 0):
                queryParams.append({"name": "agentDeployId", "value": ",".join(execution_query_model.agent_deploy_id_list)})

            if (execution_query_model.user_id_list != None and len(execution_query_model.user_id_list) > 0):
                queryParams.append({"name": "agentExecCliUserId", "value": json.dumps(execution_query_model.user_id_list)})

            if (execution_query_model.session_id_list != None and len(execution_query_model.session_id_list) > 0):
                queryParams.append({"name": "agentExecCliSessionId", "value": json.dumps(execution_query_model.session_id_list)})

            if (execution_query_model.channel_id_list != None and len(execution_query_model.channel_id_list) > 0):
                queryParams.append({"name": "agentExecCliChannel", "value": json.dumps(execution_query_model.channel_id_list)})

            if (execution_query_model.environment_id != None):
                queryParams.append({"name": "environmentId", "value": execution_query_model.environment_id})

            if (execution_query_model.isTestPlan != None):
                queryParams.append({"name": "agentExecIsTest", "value": execution_query_model.isTestPlan})

            if (execution_query_model.finished != None):
                queryParams.append({"name": "agentExecSuccessfull", "value": execution_query_model.finished})

            if (execution_query_model.eval_id_user != None and len(execution_query_model.eval_id_user.eval_id) > 0):
                queryParams.append({"name": "evalIdUser", "value": ",".join(execution_query_model.eval_id_user.eval_id)})
                if (execution_query_model.eval_id_user.include == False):
                    excludeFilter.append("EVALIDUSER")

            if (execution_query_model.eval_id_internal != None and len(execution_query_model.eval_id_internal.eval_id) > 0):
                queryParams.append({"name": "evalIdInternal", "value": ",".join(execution_query_model.eval_id_internal.eval_id)})
                if (execution_query_model.eval_id_internal.include == False):
                    excludeFilter.append("EVALIDINTERNAL")

            if (execution_query_model.eval_id_auto != None and len(execution_query_model.eval_id_auto.eval_id) > 0):
                queryParams.append({"name": "evalIdAuto", "value": ",".join(execution_query_model.eval_id_auto.eval_id)})
                if (execution_query_model.eval_id_auto.include == False):
                    excludeFilter.append("EVALIDAUTO")

            if (execution_query_model.data_filter_list != None and len(execution_query_model.data_filter_list) > 0):
                dataValue = json.dumps(list(map(lambda x: x.to_json(), execution_query_model.data_filter_list)))
                #to base64
                bytes = dataValue.encode('ascii')
                dataValue64 = base64.b64encode(bytes).decode('ascii')

                queryParams.append({"name": "dataFilter", "value": dataValue64})

            if (len(excludeFilter) > 0):
                queryParams.append({"name": "exclude", "value": ";".join(excludeFilter)})

            self.message = "starting postRequest"
            huemul_response = HuemulConnection(connect_object=self.connect_object).get_request(
                route = "agentExec/v1/",
                queryParams=queryParams
            )

            #get status from connection
            self.message = "starting fromResponseProvider"
            self.from_response_provider(huemul_response_provider = huemul_response)
            if (self.isSuccessful):
                self.data = [] if len(huemul_response.data_raw) == 0 else list(map(lambda x: ExecutionResponseModel(**x) ,huemul_response.data_raw))
        except Exception as e:
            self.errors.append(
                HuemulResponseError(errorId = "APP-101", errorTxt = str(e))
            )

        return self


import requests
from enola.base.common.huemul_http_info import HuemulHttpInfo
from enola.base.common.huemul_response_provider import HuemulResponseProvider
from enola.base.common.huemul_response_error import HuemulResponseError
import json

from enola.base.connect import Connect

class HuemulConnection:
    def __init__(self, connect_object: Connect):
        self.connectObject = connect_object
        #self.httpClient: CloseableHttpClient = HttpClients.createDefault

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

    #
    # get HTTP call to post method
    # @param route url
    # @param data info to be sent to post method
    # @return
    #
    def auth_request(self, route, data, org_id):
        if (self.connectObject.huemul_common.get_service_url() == ""):
            raise NameError('API Url null or empty')

        httpInfo = HuemulHttpInfo("", -1)

        try:
            # create object
            uriFinal = self.connectObject.huemul_common.get_service_url() + route

            #add header
            headers = self.get_header_for_auth({
                "authorization" : "Basic " + data,
                "orgId": org_id,
            })

            payload = "".format("")
            httpInfo = requests.request("POST", uriFinal, data=payload, headers=headers)
            # print(response.text)
        except Exception as e:
            print(e)
            

        value = self._get_response(httpInfo)
        return value
        

    #
    # get HTTP call to get method
    # @param route url
    # @param queryParams query params
    # @param headerParams header params
    # @return
    #
    def get_request(self, route, queryParams = [], headerParams = None):
        huemul_response_on_error = HuemulResponseProvider() #used only if error exists

        if (self.connectObject.huemul_common.get_service_url() == ""):
            raise NameError('API Url null or empty')

        routeParams = ""
        for i in range(len(queryParams)):
            routeParams = routeParams + ("?" if i == 0 else "&") + str(queryParams[i].get("name")) + "=" + str(queryParams[i].get("value"))
        
        httpInfo = HuemulHttpInfo("", -1)

        try:
            # create object
            uriFinal = self.connectObject.huemul_common.get_service_url() + route + routeParams

            #add header
            headers = self.get_header(headerParams=headerParams)

            httpInfo = requests.request("GET", uriFinal, headers=headers)
            # print(response.text)

            value = self._get_response(httpInfo)
            return value
        except requests.exceptions.HTTPError as http_err:
            error_code = httpInfo.status_code if 'httpInfo' in locals() else None
            print(f'HTTP error occurred: {http_err}')  # Nombre del error y descripción
            print(f'HTTP status code: {error_code}')
            
            huemul_response_on_error.errors.append(HuemulResponseError(errorId = error_code, errorTxt = str(http_err)))
        except requests.exceptions.ConnectionError as conn_err:
            print(f'Connection error occurred: {conn_err}')  # Nombre del error y descripción
            huemul_response_on_error.errors.append(HuemulResponseError(errorId = "ConnectionError", errorTxt = str(conn_err)))
        except requests.exceptions.Timeout as timeout_err:
            print(f'Timeout error occurred: {timeout_err}')  # Nombre del error y descripción
            huemul_response_on_error.errors.append(HuemulResponseError(errorId = "Timeout", errorTxt = str(timeout_err)))
        except requests.exceptions.RequestException as req_err:
            print(f'An error occurred: {req_err}')  # Nombre del error y descripción
            huemul_response_on_error.errors.append(HuemulResponseError(errorId = "RequestException", errorTxt = str(req_err)))
        except Exception as e:
            print(e)
            huemul_response_on_error.errors.append(HuemulResponseError(errorId = "connection_other", errorTxt = str(e)))

        return huemul_response_on_error
        

    #
    # get HTTP call to post method
    # @param route url
    # @param queryParams query params
    # @param data info to be sent to post method
    # @return
    #
    def post_request(self, route, data, queryParams = [], headerParams = None):
        if (self.connectObject.huemul_common.get_service_url() == ""):
            raise NameError('API Url null or empty')

        routeParams = ""
        for i in range(len(queryParams)):
            routeParams = routeParams + ("?" if i == 0 else "&") + str(queryParams[0].name) + "=" + str(queryParams[0].value)
        
        httpInfo = HuemulHttpInfo("", -1)

        try:
            # create object
            uriFinal = self.connectObject.huemul_common.get_service_url() + route + routeParams

            #add header
            headers = self.get_header(headerParams=headerParams)

            payload = data #"".format("")
            httpInfo = requests.request("POST", uriFinal, data=payload, headers=headers)
            # print(response.text)

            value = self._get_response(httpInfo)
            return value
        except Exception as e:
            print(e)
            huemulResponse = HuemulResponseProvider()
            huemulResponse.errors.append(HuemulResponseError(errorId = "post_error", errorTxt = str(e)))

            return huemulResponse

            
        

    #
    # get HTTP call to post method
    # @param route url
    # @param queryParams query params
    # @param data info to be sent to put method
    # @return
    #
    def put_request(self, route, data, queryParams = []):
        if (self.connectObject.huemul_common.get_service_url() == ""):
            raise NameError('API Url null or empty')

        routeParams = ""
        for i in range(len(queryParams)):
            routeParams = routeParams + ("?" if i == 0 else "&") + str(queryParams[0].name) + "=" + str(queryParams[0].value)
        
        httpInfo = HuemulHttpInfo("", -1)

        try:
            # create object
            uriFinal = self.connectObject.huemul_common.get_service_url() + route + routeParams

            #add header
            headers = self.get_header(headerParams=None)

            payload = data #"".format("")
            httpInfo = requests.request("PUT", uriFinal, data=payload, headers=headers)
            # print(response.text)
        except Exception as e:
            print(e)
            
        value = self._get_response(httpInfo)
        return value

    #
    # common header for each http method
    # headerParams: Dictionary
    # @return Dictionary
    #
    def get_header_for_auth(self, headerParams):
        dataToReturn = {
            "Accept" : "application/json",
            "content-type" : "application/json",
            "huemul-client-language" : "PYTHON",
            "huemul-client-version" : "1.0",
            "huemul-client-app" : "SERVER",
            "huemul-client-info" : "",
        }

        if (headerParams != None):
            dataToReturn.update(headerParams)

        return dataToReturn


    #
    # common header for each http method
    # headerParams: Dictionary
    # @return Dictionary
    #
    def get_header(self, headerParams):
        dataToReturn = {
            "Accept" : "application/json",
            "content-type" : "application/json",
            "huemul-client-language" : "PYTHON",
            "huemul-client-version" : "1.0",
            "huemul-client-app" : "SERVER",
            "huemul-client-info" : "",
            "authorization": "Bearer " + self.connectObject.huemul_common.get_token_id(),
            "orgId" : self.connectObject.huemul_common.get_org_id()
        }

        if (headerParams != None): 
            dataToReturn.update(headerParams)

        return dataToReturn


    # transform data from api to response
    # response: HuemulHttpInfo
    # return HuemulResponseProvider
    def _get_response(self, response):
        huemulResponse = HuemulResponseProvider()

        try:
            if (response.text != ""):
                dataFromJson = json.loads(response.text)
                huemulResponse.fromDict(**dataFromJson)
            else:
                huemulResponse.isSuccessful = False
                huemulResponse.errors.append(HuemulResponseError(errorId = "getResponseError", errorTxt = f'status {response.status_code}: response.text is empty'))
                huemulResponse.httpStatusCode = response.status_code
                huemulResponse.message = response.reason
            
            #huemulResponse.fromDict(**dataFromJson) #HuemulResponseProvider.fromJson(response.text)
        except Exception as e:
            #huemulResponse.isSuccessful = false;
            huemulResponse.errors.append(HuemulResponseError(errorId = "getResponseError", errorTxt = str(e)))
        
        return huemulResponse


from enola.base.connect import Connect
from enola.base.internal.evaluation.enola_evaluation_provider import EnolaEvaluationProvider
from enola.enola_types import EvaluationModel

class EnolaEvaluationBloc():
    #
    # start enolaAgentCreate
    # @param AgentModel AgentModel
    # @return HuemulResponseBloc[EnolaAgentResponseModel]
    #
    def enola_evaluation_create(self, evaluation_model: EvaluationModel, connect_object: Connect):
        (continue_in_loop) = True
        attempt = 0
        #result = HuemulResponseToBloc(connectObject=connectObject)

        while ((continue_in_loop)):
            result = EnolaEvaluationProvider(connect_object=connect_object).evaluation_create(
                    evaluation_model=evaluation_model
            )
            attempt +=1
            (continue_in_loop) = result.analyze_errors(attempt)
        
        return result


from enola.base.common.auth.auth_model import AuthModel
from enola.base.common.huemul_common import HuemulCommon
from enola.base.common.huemul_error import HuemulError
from enola.base.common.huemul_logging import HuemulLogging

# authData: AuthModel
class Connect:
    def __init__(self, auth_data: AuthModel):
        self.authData = auth_data
        self.huemul_logging = HuemulLogging()
        self.huemul_logging.log_message_info(message = "WELCOME to Enola...")

        self._can_execute = False
        self._is_open = True
        self._error_message = ""
        self.others_params = []
        self.control_class_name = "" #: String = Invoker(1).getClassName.replace("$", "")
        self.control_method_name = "" #: String = Invoker(1).getMethodName.replace("$", "")
        #val controlFileName: String = Invoker(1).getFileName.replace("$", "")

        #create error and common object
        self.control_error = HuemulError(org_id=auth_data.org_id, huemul_logging=self.huemul_logging)
        self.huemul_common = HuemulCommon()
        
        #valida que jwtToken no sea nulo
        if (auth_data.jwt_token is None):
            self.huemul_logging.log_message_error(message = "jwtToken is null")
            self._error_message = "jwtToken is null"
            self._is_open = False
            return

        #store credentials and token
        self.huemul_common.set_org_id(auth_data.org_id)
        self.huemul_common.set_consumer_id(auth_data.consumer_id)
        self.huemul_common.set_consumer_secret(auth_data.consumer_secret)
        self.huemul_common.set_application_name(auth_data.application_name)
        self.huemul_common.set_jwt_token(auth_data.jwt_token)
        self.huemul_common.set_token_id(self.authData.jwt_token)
        self.huemul_common.set_service_url(self.authData.url_service)
        

        self.huemul_logging.log_message_info(message = "authorized...")
        self.can_execute = True
        self.huemul_logging.log_message_info(message = "STARTED!!!")

        ### END START
        
    def is_open(self):
        return self._is_open

    #/************************************************************************************/
    #/******************  R E S U L T S    ***********************************************/
    #/************************************************************************************

    
    

    #/************************************************************************************/
    #/******************  U T I L   F U N C T I O N S    *********************************/
    #/************************************************************************************/

    #
    # true for execute, false can't execute
    # @return
    #
    def can_execute(self):
        return self._can_execute

    #
    # return error message
    # @return
    #
    def get_error_message(self):
        return self._error_message
    
    


from ast import Try
import json
import time
from warnings import catch_warnings
from enola.base.common.huemul_connection import HuemulConnection
from enola.base.common.huemul_response_provider import HuemulResponseProvider
from enola.base.connect import Connect

#
# @author Sebastián Rodríguez Robotham
# base class to create, get, getAll methods exposed to user
# @tparam T class Model
#


class HuemulResponseToBloc(HuemulResponseProvider):
    def __init__(self, connect_object: Connect, **args):
        self.data = "" #huemulResponseProvider.dataRaw
        self.connect_object = connect_object
        self.isSuccessful = False
        # status code: 200 OK, 500 error, etc
        self.httpStatusCode = ""
        # text to client
        self.message = "Not started"
        self.startDate = ""
        self.endDate = ""
        self.elapsedTimeMS = -1
        self.transactionId = ""
        # api response version
        self.apiVersion = ""

        #error detail
        self.errors = []
        #data detail
        self.dataRaw = ""
        #extra info detail
        self.extraInfoRaw = ""

        #print("paso 100")
        if (len(args) == 1 and "huemulResponseProvider" in args):
            #print("paso 200")
            self.from_response_provider(args["huemulResponseProvider"])

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)


    def from_response_provider(self, huemul_response_provider: HuemulResponseProvider):
        #print("paso 300")
        self.data = huemul_response_provider.data_raw #huemulResponseProvider.dataRaw
        self.isSuccessful = huemul_response_provider.isSuccessful
        # status code: 200 OK, 500 error, etc
        self.httpStatusCode = huemul_response_provider.httpStatusCode
        # text to client
        self.message = huemul_response_provider.message
        self.startDate = huemul_response_provider.startDate
        self.endDate = huemul_response_provider.endDate
        self.elapsedTimeMS = huemul_response_provider.elapsedTimeMS
        self.transactionId = huemul_response_provider.transactionId
        # api response version
        self.apiVersion = huemul_response_provider.apiVersion

        #error detail
        self.errors = huemul_response_provider.errors
        #data detail
        self.dataRaw = huemul_response_provider.data_raw
        #extra info detail
        self.extraInfoRaw = huemul_response_provider.extraInfoRaw
        #print("paso 400")

    #
    #analyze error and determine attempts strategy
    # @param result create/get/getAll response (HuemulResponseBloc type)
    # @param attempt attempt number
    # @return Boolean
    #
    def analyze_errors(self, attempt):
        #print("paso 500")
        continueInLoop = True

        if (self.isSuccessful):
            #all right, exit
            continueInLoop = False
        elif (attempt < self.connect_object.huemul_common.get_total_attempt()):
            #send errors
            self.connect_object.huemul_logging.log_message_info(f"Error in step {self.message}")
            #self.connectObject.huemulLogging.logMessageInfo(str(self.errors))

            try:
                #errorText = ';'.join(map(lambda x: str(x["errorId"]) + ": " + x["errorTxt"],self.errors))
                errorText = self.message if (len(self.errors) == 0) else ';'.join(map(lambda x: str(x["errorId"]) + ": " + x["errorTxt"],self.errors))
            except Exception as e:
                try:
                    #errorText = ';'.join(map(lambda x: str(x.errorId) + ": " + x.errorTxt,self.errors))
                    errorText = self.message if (len(self.errors) == 0) else ';'.join(map(lambda x: str(x.errorId) + ": " + x.errorTxt,self.errors))
                except Exception as e:
                    errorText = "error try to catch error: " + str(e)

            self.connect_object.huemul_logging.log_message_error("errors details: " + errorText)
            self.connect_object.huemul_logging.log_message_error("errors transaction-id: " + self.transactionId)
            #wait from second attempt
            if (attempt > 1):
                # wait 10 seconds and try to call again
                self.connect_object.huemul_logging.log_message_error("waiting 5 seconds.....")
                time.sleep(5)
            

            #get all possible errors
            try:
                connectionError = len(list(filter(lambda x: x.errorId == "APP-101" or x.errorId == "ConnectionError", self.errors)))
            except Exception as e:
                connectionError = len([error for error in self.errors if error.get("errorId") == "APP-101"])
            connectionError = -1 if connectionError == 0 else connectionError

            try:
                unAuthorizedError = len(list(filter(lambda x: x["errorId"] == "2040", self.errors)))
            except Exception as e:
                unAuthorizedError = len([error for error in self.errors if error.get("errorId") == "2040"])
            unAuthorizedError = -1 if unAuthorizedError == 0 else unAuthorizedError

            try:
                forbiddenError = len(list(filter(lambda x: x["errorId"] == "2030", self.errors)))
            except Exception as e:
                forbiddenError = len([error for error in self.errors if error.get("errorId") == "2030"])
            forbiddenError = -1 if forbiddenError == 0 else forbiddenError

            if (forbiddenError > 0):
                self.connect_object.huemul_logging.log_message_error("forbidden")

            if (unAuthorizedError > 0):
                self.connect_object.huemul_logging.log_message_error("attempt " + str(attempt + 1) + " of " + str(self.connect_object.huemul_common.get_total_attempt()))
                #check if error = unauthorized, try to login again
                continueInLoop = True
            elif (connectionError > 0):
                self.connect_object.huemul_logging.log_message_error("attempt " + str(attempt + 1) + " of " + str(self.connect_object.huemul_common.get_total_attempt()))
                #raised error from HuemulConnection method
                continueInLoop = True
            else:
                #unknown error (maybe data), exit and return error
                continueInLoop = False

        else:
            continueInLoop = False


        return continueInLoop
    


from enola.base.common.huemul_functions import HuemulFunctions
from enola.enola_types import ApiDataModel, ErrOrWarnKind, ErrorOrWarnModel, ErrorType, FileInfoModel, Info
from enum import Enum

class StepCost:
    def __init__(self):
        self.token_input = 0
        self.token_output = 0
        self.token_total = 0
        self.videos = 0
        self.audio = 0
        self.images = 0
        self.docs = 0
        self.infra = 0
        self.others = 0
        self.total = 0

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class StepVideo:
    def __init__(self):
        self.num_videos = 0
        self.size_videos = 0
        self.sec_videos = 0

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class StepAudio:
    def __init__(self):
        self.size_audio = 0
        self.num_audio = 0
        self.sec_audio = 0

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class StepImage:
    def __init__(self):
        self.num_images = 0
        self.size_images = 0

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class StepDoc:
    def __init__(self):
        self.num_docs = 0
        self.num_pages = 0
        self.size_docs = 0
        self.num_char = 0

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class StepToken:
    def __init__(self):
        self.num_char = 0
        self.token_input = 0
        self.token_output = 0
        self.token_total = 0

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class Step:
    def __init__(self, name: str, message_input: str = ""):
        self.hf = HuemulFunctions()
        self.name = name
        self.enola_id = ""
        self.agent_deploy_id = ""
        self.step_id = ""
        self.message_input = message_input
        self.message_output = ""
        self.num_iterations = 0
        self.step_api_code = ""
        self.step_id_prev = ""
        self.date_start = self.hf.get_date_for_api()
        self.date_end = self.date_start
        #self.result_list = []
        self.agent_data_list = [] #only for first step
        self.errOrWarn_list = []
        self.extra_info_list = []
        self.file_info_list = []
        self.step_api_data_list = []
        self.step_type:StepType = StepType.OTHER

        self.successfull = False
        self.num_errors = 0
        self.num_warnings = 0

        self.video = StepVideo()
        self.audio = StepAudio()
        self.image = StepImage()
        self.doc = StepDoc()
        self.token = StepToken()
        self.cost = StepCost()
        self.income_total = 0
        self.duration_in_ms = 0        

    def add_api_data(self, bodyToSend: str, payloadReceived: str, name: str, method: str, url: str, description: str = "", headerToSend: str = ""):
        self.step_api_data_list.append(ApiDataModel(name=name, method=method, url=url, body=bodyToSend, header=headerToSend, payload=payloadReceived, description=description)) 
        #{"body": bodyToSend, "header": headerToSend, "payload": payloadReceived})

    def add_file_link(self, name: str, url: str, type: str, size_kb: int, description: str = ""):
        self.file_info_list.append(FileInfoModel(name=name, url=url, type=type, sizeKb=size_kb, description=description))
        #{"name": name, "url": url, "type": type, "size": size})

    def add_tag(self, key: str, value):
        self.extra_info_list.append(Info(type="tag", key=key, value=value))
        
    def add_extra_info(self, key: str, value):
        self.extra_info_list.append(Info(type="info", key=key, value=value))

    def add_error(self, id: str, message: str, kind: ErrOrWarnKind):
        self.num_errors += 1
        self.errOrWarn_list.append(ErrorOrWarnModel(id=id, message=message, error_type=ErrorType.ERROR, kind=kind))

    def add_warning(self, id: str, message: str, kind: ErrOrWarnKind):
        self.num_warnings += 1
        self.errOrWarn_list.append(ErrorOrWarnModel(id=id, message=message, error_type=ErrorType.ERROR, kind=kind))

    def to_json(self):
        return {
            "stepId": self.step_id,
            "stepIdPrev": self.step_id_prev,
            "stepDateStart": self.date_start,
            "stepDateEnd": self.date_end,
            "agentDeployId": self.agent_deploy_id,
            "agentExecName": self.name,
            "agentExecDurationMs": self.duration_in_ms,
            "agentExecCliCodeApi": self.step_api_code,
            "agentExecSuccessfull": self.successfull,
            "agentExecNumErrors": self.num_errors,
            "agentExecNumWarnings": self.num_warnings,
            "agentExecNumVideos": self.video.num_videos,
            "agentExecSecVideos": self.video.sec_videos,
            "agentExecSizeAudio": self.video.size_videos,
            "agentExecNumAudio": self.audio.num_audio,
            "agentExecSecAudio": self.audio.sec_audio,
            "agentExecSizeVideos": self.video.size_videos,
            "agentExecNumImages": self.image.num_images,
            "agentExecSizeImages": self.image.size_images,
            "agentExecNumDocs": self.doc.num_docs,
            "agentExecNumPages": self.doc.num_pages,
            "agentExecSizeDocs": self.doc.size_docs,
            "agentExecNumChar": self.doc.num_char + self.token.num_char,
            "agentExecTokenInput": self.token.token_input,
            "agentExecTokenOutput": self.token.token_output,
            "agentExecTokenTotal": self.token.token_total,
            "agentExecCostTokenInput": self.cost.token_input,
            "agentExecCostTokenOutput": self.cost.token_output,
            "agentExecCostTokenTotal": self.cost.token_total,
            "agentExecCostVideos": self.cost.videos,
            "agentExecCostAudio": self.cost.audio,
            "agentExecCostImages": self.cost.images,
            "agentExecCostDocs": self.cost.docs,
            "agentExecCostInfra": self.cost.infra,
            "agentExecCostOthers": self.cost.others,
            "agentExecCostTotal": self.cost.total,
            "agentExecIncomeTotal": self.income_total,

            "agentExecMessageInput": self.message_input,
            "agentExecMessageOutput": self.message_output,
            "agentExecCliNumIter": self.num_iterations,

            "agentData": list(map(lambda x: x.to_json(), self.agent_data_list)),
            "errorOrWarning": list(map(lambda x: x.to_json(), self.errOrWarn_list)),
            "extraInfo": list(map(lambda x: x.to_json(), self.extra_info_list)),
            "fileInfo": list(map(lambda x: x.to_json(), self.file_info_list)),
            "stepApiData": list(map(lambda x: x.to_json(), self.step_api_data_list)),
        }

    def __str__(self):
        return f'Step: {self.description}, Duration: {self.duration} seconds'
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)


class StepType(Enum):
    TOKEN = "TOKEN"
    VIDEO = "VIDEO"
    AUDIO = "AUDIO"
    IMAGE = "IMAGE"
    DOCUMENT = "DOCUMENT"
    OTHER = "OTHER"



from enola.base.common.huemul_functions import HuemulFunctions
from enola.base.internal.tracking.enola_tracking import create_tracking
from enola.step import Step, StepType
from enola.base.common.auth.auth_model import AuthModel
from enola.enola_types import EnolaSenderModel, KindType, DataListModel, DataType, ErrOrWarnKind, Info, TokenInfo, TrackingModel
from enola.base.connect import Connect


#def add_one(number):
#    return number + 1

class Tracking:
    def __init__(self, token, name, app_id=None, user_id=None, session_id=None, channel_id=None, ip=None, code_api=None, is_test=False, message_input: str = "", enola_id_prev:str = "", app_name:str="", user_name:str="", session_name:str="", channel_name:str=""):
        """
        Start tracking Execution

        token: jwt token, this is used to identify the agent, request from Admin App
        name: name of this execution
        message_input: message received from user or to explain the execution
        app_id: id of app, this is used to identify the app who is calling
        user_id: id of external user, this is used to identify the user who is calling
        session_id: id of session of user or application, this is used to identify the session who is calling
        channel_id: web, chat, whatsapp, etc, this is used to identify the channel who is calling
        ip: ip of user or application, this is used to identify the ip who is calling
        code_api: code of api, this is used to identify the api who is calling
        is_test: true if this call is for testing purposes
        enola_id_prev: id of previous call, this is used to link agents sequence
        """
        self.name = name
        self.enola_id_prev = enola_id_prev
        self.enola_id = "" #se obtiene al finalizar la ejecución
        self.agent_deploy_id = ""
        self.message_input = message_input
        self.message_output = ""
        self.num_iteratons = 0
        self.hf = HuemulFunctions()
        self.url_evaluation_post = None
        self.url_evaluation_def_get = None

        #this execution information
        self.tracking_status = ""
        #Connection data

        #decodificar jwt
        self.token_info = TokenInfo(token=token)

        if (self.token_info.is_service_account == False):
            raise Exception("This token is not a service account. Only service accounts can execute tracking")

        if not self.token_info.agent_deploy_id:
            raise Exception("agentDeployId is empty.")

        if (self.token_info.service_account_can_tracking == False):
            raise Exception("This service account can't execute tracking")
        

        self.agent_deploy_id = self.token_info.agent_deploy_id
        self.connection = Connect(AuthModel(jwt_token=token, url_service=self.token_info.service_account_url, org_id=self.token_info.org_id))
        
        #user information
        self.enola_sender = EnolaSenderModel(app_id=app_id, app_name=app_name, user_id=user_id, user_name=user_name, session_id=session_id, session_name=session_name, channel_id=channel_id, channel_name=channel_name, ip=ip, code_api=code_api)

        #current date
        self.date_start = self.hf.get_date_for_api()
        
        #if is empty or not exist assign false
        self.is_test = is_test
        
        #save steps and informations
        self.step_list = []
        self.steps = 0
        self.first_step = self.new_step(self.name, message_input= self.message_input)

    ########################################################################################
    ###############    A G E N T   M E T H O D S     #######################################
    ########################################################################################


    def add_data_received(self, name:str, data, type:DataType):
        """
        add data received from user
        """
        self.first_step.agent_data_list.append(DataListModel(value=data, name=name, data_type=type, kind=KindType.RECEIVER)) 

    def add_data_send(self, name:str, data, type:DataType):
        """
        add data to send to user
        """
        self.first_step.agent_data_list.append(DataListModel(value=data, name=name, data_type=type, kind=KindType.SENDER)) 
        
    def add_custom_info(self, key, value):
        """
        add custom information to tracking
        """
        self.first_step.info_list.append(Info(key, value))

    def add_file_link(self, name: str, url: str, type: str, size_kb: int):
        """
        add file link to tracking
        """
        self.first_step.add_file_link(name=name, url=url, type=type, size_kb=size_kb)

    def add_tag(self, key: str, value):
        """
        add tag to tracking, this tag is used to search in Enola App
        """
        self.first_step.add_tag(key=key, value=value)

    def add_extra_info(self, key: str, value):
        """
        add extra information to tracking, this can be used to test or debug
        """
        self.first_step.add_extra_info(key=key, value=value)

    def add_error(self, id: str, message: str, kind: ErrOrWarnKind):
        """
        register error to tracking
        """
        self.first_step.add_error(id=id, message=message, kind=kind)

    def add_warning(self, id: str, message: str, kind: ErrOrWarnKind):
        """
        register warning to tracking
        """
        self.first_step.add_warning(id=id, message=message, kind=kind)


    def execute(self, successfull: bool, message_output: str ="", num_iteratons: int = 0) -> bool:
        """
        register tracking in Enola server
        successfull: true for your Agent execution OK, false for error in your Agent execution
        message_output: message to user or to explain the execution results
        num_iteratons: number of iterations

        """
        self.date_end = self.hf.get_date_for_api()
        self.duration_in_ms = self.hf.get_dif_ms(self.date_start, self.date_end)
        self.first_step.num_iterations  = num_iteratons
        self.close_step_others(step=self.first_step, successfull=successfull, others_cost=0, step_id="AGENT", message_output=message_output)

        #register in server
        print(f'{self.name}: sending to server... ')
        tracking_model = TrackingModel(
            enola_id_prev = self.enola_id_prev,
            enola_sender= self.enola_sender,

            isTest=self.is_test, 
            step_list=self.step_list, 
            steps=self.steps
        )

        #print("paso 10")
        enola_result = create_tracking(tracking_model=tracking_model, connection=self.connection, raise_error_if_fail=True)
        #show results
        if (enola_result.successfull):
            #obtiene url para evaluación
            #obtiene id de enola
            self.enola_id = enola_result.enola_id
            self.agent_deploy_id = enola_result.agent_deploy_id
            self.url_evaluation_post = enola_result.url_evaluation_post
            self.url_evaluation_def_get = enola_result.url_evaluation_def_get

            print(f'{self.name}: finish OK! ')

            return True
        else:
            print(f'{self.name}: finish with error: {enola_result.message}')
            self.tracking_status = enola_result.message

            return False

        


    ########################################################################################
    ###############    S T E P   I N F O     ###############################################
    ########################################################################################


    def new_step(self, name: str, message_input: str = ""):
        """
        start new step
        name: name of this step
        message_input: message received from user or to explain the execution
        """
        #current_step = 
        self.steps += 1
        return Step(name=name, message_input=message_input)

    def close_step_token(self, step: Step, successfull: bool, message_output: str ="", token_input_num: int=0, token_output_num: int=0, token_total_num: int=0, token_input_cost: float=0, token_output_cost: float=0, token_total_cost: float=0, enola_id: str="", agent_deploy_id: str="", step_id:str=""):
        """
        close step with token information
        enola_id: If this step was a call to another Enola agent, whether from your own company or another, this is the ID of that agent
        agent_deploy_id: include this if you want to link this step to another agent of another company
        step_id: id of this step, you can use it to link with external calls
        message_output: message to user or to explain the execution results
        """
        step.enola_id = enola_id
        step.agent_deploy_id = agent_deploy_id
        step.step_id = step_id
        step.message_output = message_output
        step.step_type = StepType.TOKEN
        step.token.token_input = token_input_num
        step.token.token_output = token_output_num
        step.token.token_total = token_total_num
        step.cost.token_input = token_input_cost
        step.cost.token_output = token_output_cost
        step.cost.token_total = token_total_cost

        step.date_end = self.hf.get_date_for_api()
        step.duration_in_ms = self.hf.get_dif_ms(step.date_start, step.date_end)
        step.successfull = successfull

        self.step_list.append(step)

    def close_step_video(self, step: Step, successfull: bool, message_output: str ="", video_num: int=0, video_sec: int=0, video_size: int=0, video_cost: float=0, enola_id_prev: str="", agent_deploy_id: str="", step_id:str=""):
        """
        close step with video information

        enola_id_prev: If this step was a call to another Enola agent, whether from your own company or another, this is the ID of that agent
        agent_deploy_id: include this if you want to link this step to another agent of another company
        step_id: id of this step, you can use it to link with external calls
        message_output: message to user or to explain the execution results
        """
        step.enola_id_prev = enola_id_prev
        step.agent_deploy_id = agent_deploy_id
        step.step_id = step_id
        step.message_output = message_output
        step.step_type = StepType.VIDEO
        step.video.num_videos = video_num
        step.video.sec_videos = video_sec
        step.video.size_videos = video_size
        step.cost.videos = video_cost
        step.date_end = self.hf.get_date_for_api()
        step.duration_in_ms = self.hf.get_dif_ms(step.date_start, step.date_end)
        step.successfull = successfull

        self.step_list.append(step)
        
    def close_step_audio(self, step: Step, successfull: bool, message_output: str ="", audio_num:int = 0, audio_sec:int = 0, audio_size:int = 0, audio_cost: float=0, enola_id_prev: str="", agent_deploy_id: str="", step_id:str=""):
        """
        close step with audio information

        enola_id_prev: If this step was a call to another Enola agent, whether from your own company or another, this is the ID of that agent
        agent_deploy_id: include this if you want to link this step to another agent of another company
        step_id: id of this step, you can use it to link with external calls
        message_output: message to user or to explain the execution results
        """
        step.enola_id_prev = enola_id_prev
        step.agent_deploy_id = agent_deploy_id
        step.step_id = step_id
        step.message_output = message_output
        step.step_type = StepType.AUDIO
        step.audio.num_audio = audio_num
        step.audio.sec_audio = audio_sec
        step.audio.size_audio = audio_size
        step.cost.audio = audio_cost
        step.date_end = self.hf.get_date_for_api()
        step.duration_in_ms = self.hf.get_dif_ms(step.date_start, step.date_end)
        step.successfull = successfull

        self.step_list.append(step)

    def close_step_image(self, step: Step, successfull: bool, message_output: str ="", image_num:int = 0, image_size:int = 0, image_cost: float = 0, enola_id_prev: str="", agent_deploy_id: str="", step_id:str=""):
        """
        close step with image information

        enola_id_prev: If this step was a call to another Enola agent, whether from your own company or another, this is the ID of that agent
        agent_deploy_id: include this if you want to link this step to another agent of another company
        step_id: id of this step, you can use it to link with external calls
        message_output: message to user or to explain the execution results
        """
        step.enola_id_prev = enola_id_prev
        step.agent_deploy_id = agent_deploy_id
        step.step_id = step_id
        step.message_output = message_output
        step.step_type = StepType.IMAGE
        step.image.num_images = image_num
        step.image.size_images = image_size
        step.cost.images = image_cost
        step.date_end = self.hf.get_date_for_api()
        step.duration_in_ms = self.hf.get_dif_ms(step.date_start, step.date_end)
        step.successfull = successfull

        self.step_list.append(step)

    def close_step_doc(self, step: Step, successfull: bool, message_output: str ="", doc_num:int=0, doc_pages:int=0, doc_size:int = 0, doc_char:int=0, doc_cost: float=0, enola_id_prev: str="", agent_deploy_id: str="", step_id:str=""):
        """
        close step with doc information

        enola_id_prev: If this step was a call to another Enola agent, whether from your own company or another, this is the ID of that agent
        agent_deploy_id: include this if you want to link this step to another agent of another company
        step_id: id of this step, you can use it to link with external calls
        message_output: message to user or to explain the execution results
        """
        step.enola_id_prev = enola_id_prev
        step.agent_deploy_id = agent_deploy_id
        step.step_id = step_id
        step.message_output = message_output
        step.step_type = StepType.DOCUMENT
        step.doc.num_docs = doc_num
        step.doc.num_pages = doc_pages
        step.doc.size_docs = doc_size
        step.doc.num_char = doc_char
        step.cost.docs = doc_cost
        step.date_end = self.hf.get_date_for_api()
        step.duration_in_ms = self.hf.get_dif_ms(step.date_start, step.date_end)
        step.successfull = successfull

        self.step_list.append(step)

    def close_step_others(self, step: Step, successfull: bool, message_output: str ="", others_cost: float=0, enola_id_prev: str="", agent_deploy_id: str="", step_id:str=""):
        """
        close step with others information

        enola_id_prev: If this step was a call to another Enola agent, whether from your own company or another, this is the ID of that agent
        agent_deploy_id: include this if you want to link this step to another agent of another company
        step_id: id of this step, you can use it to link with external calls
        message_output: message to user or to explain the execution results
        """
        step.enola_id_prev = enola_id_prev
        step.agent_deploy_id = agent_deploy_id
        step.step_id = step_id
        step.message_output = message_output
        step.step_type = StepType.OTHER
        step.cost.others = others_cost
        step.date_end = self.hf.get_date_for_api()
        step.duration_in_ms = self.hf.get_dif_ms(step.date_start, step.date_end)
        step.successfull = successfull

        self.step_list.append(step)

    

    def __str__(self):
        return f'Agent/Model: {self.name}, Steps: {self.steps}'
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)





import base64
from enola.base.common.auth.auth_service_model import AuthServiceModel
from enola.base.common.huemul_connection import HuemulConnection
from enola.base.common.huemul_response_error import HuemulResponseError
from enola.base.common.huemul_response_to_bloc import HuemulResponseToBloc

class AuthServiceProvider(HuemulResponseToBloc):
    
    #
    # create new element
    # @param consumer_id consumer_id: String
    # @param consumerSecret consumerSecret: String
    # @param org_id orgIorg_idd: String
    # @param applicationName applicationName: String
    #
    def authSignInService(self, consumer_id, consumer_secret, org_id, application_name):
        try:
            dataIn = consumer_id + ":" + consumer_secret
            bytes = dataIn.encode('ascii') #.getBytes(StandardCharsets.UTF_8)
            base64Str = base64.b64encode(bytes).decode('ascii')

            huemulResponse = HuemulConnection(connect_object=self.connect_object).auth_request(
                route = "authService/v1/sign-in-service/",
                data = base64Str,
                org_id = org_id
            )

            #get status from connection
            self.from_response_provider(huemul_response_provider = huemulResponse)
            if (self.isSuccessful):
                self.data = [] if len(huemulResponse.data_raw) == 0 else list(map(lambda x: AuthServiceModel(**x) ,huemulResponse.data_raw))
        except Exception as e:
            self.errors.append(
                HuemulResponseError(errorId = "APP-101", errorTxt = str(e))
            )

        return self


        


from enola.base.connect import Connect
from enola.base.internal.executions.enola_execution_bloc import EnolaExecutionBloc
from enola.enola_types import ExecutionModel, ExecutionQueryModel


def get_execution(execution_query_model: ExecutionQueryModel, connection: Connect, raise_error_if_fail = True) -> ExecutionModel:
    if (not connection.can_execute):
        connection.huemul_logging.log_message_error(message = "can't execute: ")
        return ExecutionModel(
                data=[],
                successfull=False,
                message="Not Executed yet, can't execute:"
            )

    connection.huemul_logging.log_message_info(message = "Running Enola Execution")

    enola_execution_result = EnolaExecutionBloc().enola_execution_get(execution_query_model=execution_query_model,connect_object=connection)
    #if error
    if (not enola_execution_result.isSuccessful):
        print(enola_execution_result)
        connection._can_execute = False
        try:
            connection._error_message = enola_execution_result.message if (len(enola_execution_result.errors) == 0) else enola_execution_result.errors[0]["errorTxt"]
        except:
            connection._error_message = enola_execution_result.message if (len(enola_execution_result.errors) == 0) else enola_execution_result.errors[0].errorTxt

        connection.huemul_logging.log_message_error(message = "error in enolaExecution: " + connection._error_message)

        if (raise_error_if_fail):
            connection.huemul_logging.log_message_error(message = "error" + connection._error_message)
            raise NameError(connection._error_message)
        else:
            connection.huemul_logging.log_message_error(message = "error" + connection._error_message)
            return ExecutionModel(
                data=[],
                successfull=False,
                message="error" + connection._error_message,
            )

    #if all ok, continue
    # connectObject._processExecStepId = enolaAgentResult.data[0].processExecStepId

    return ExecutionModel(
        data=enola_execution_result.data,
        successfull=enola_execution_result.isSuccessful,
        message=enola_execution_result.message,
    )


from enola.base.common.auth.auth_model import AuthModel
from enola.base.common.huemul_functions import HuemulFunctions
from enola.base.connect import Connect
from enola.base.internal.executions.enola_execution import get_execution
from enola.enola_types import Environtment, ExecutionEvalFilter, ExecutionModel, ExecutionQueryModel, TokenInfo

class GetExecutions:
    def __init__(self, token: str, raise_error_if_fail = True):
        """
        Start Executions Execution

        token: jwt token, this is used to identify the agent, request from Admin App
        """
        self.raise_error_if_fail = raise_error_if_fail
        self.num_rows_acum = 0
        self.num_rows = 0
        self.limit = 0
        self.page_number = 0
        self.hf = HuemulFunctions()
        #Connection data

        #get token info
        self.token_info = TokenInfo(token=token)

        if (self.token_info.is_service_account == True and self.token_info.service_account_can_get_executions == False):
            raise Exception("Service Account Token is not allowed to get executions")

        self.connection = Connect(AuthModel(jwt_token=token, url_service=self.token_info.service_account_url_backend, org_id=self.token_info.org_id))

    def get_next_page(self):
        if (self.continue_execution == False):
            raise Exception("No more data to show.")
        
        self.execution_query_model.page_number += 1
        enola_result = self.__run_query()

        #show results
        return enola_result
        

    def get_items(self, 
                 date_from:str,
                 date_to:str,
                 chamber_id_list:list = [], 
                 agent_id_list:list = [], 
                 agent_deploy_id_list:list = [],
                 user_id_list:list = [],
                 session_id_list:list = [],
                 channel_id_list:list = [],
                 data_filter_list:list = [],
                 eval_id_user: ExecutionEvalFilter = None,
                 eval_id_internal: ExecutionEvalFilter = None,
                 eval_id_auto: ExecutionEvalFilter = None,
                 environment_id:Environtment = None,
                 is_test_plan: bool= None,
                 finished:bool = None,
                 limit:int=100, 
                 page_number:int=1, 
                 include_tags:bool=False,
                 include_data:bool=False,
                 include_errors:bool=False,
                 include_evals:bool=False,
                 ) -> ExecutionModel:
        """
        Get Items by Chamber

        date_from: str, date from
        date_to: str, date to
        chamber_id: list, chamber id
        agent_id: list, agent id
        agent_deploy_id: list, agent deploy id
        user_id: list, user id
        session_id: list, session id
        channel_id: list, channel id
        data_filter: list, data filter
        eval_id_user: ExecutionEvalFilter, eval id user
        eval_id_internal: ExecutionEvalFilter, eval id internal
        eval_id_auto: ExecutionEvalFilter, eval id auto
        environment_id: Environtment, environment id
        is_test_plan: bool, is test plan
        finished: bool, finished
        limit: int, limit
        page_number: int, page number
        include_tags: bool, include tags
        include_data: bool, include data
        include_errors: bool, include errors
        include_evals: bool, include evals
        """

        if (self.token_info.agent_deploy_id != None and self.token_info.agent_deploy_id != "" and self.token_info.agent_deploy_id != "0"):
            if (len(agent_deploy_id_list) > 1):
                raise Exception("Service Account Token is not allowed to access more than one agent_deploy_id", self.token_info.agent_deploy_id)
            if (len(agent_deploy_id_list) == 1):
                if (self.token_info.agent_deploy_id != agent_deploy_id_list[0]):
                    raise Exception("Service Account Token is not allowed to access only one agent_deploy_id", self.token_info.agent_deploy_id)
            else:
                agent_deploy_id_list = [self.token_info.agent_deploy_id]


        if (chamber_id_list == [] and agent_id_list == [] and agent_deploy_id_list == []):
            raise Exception("chamber_id or agent_id or agent_deploy_id must be filled.")
    
        self.num_rows_acum = 0
        self.limit = limit
        self.page_number = page_number

        self.execution_query_model = ExecutionQueryModel(
            date_from=date_from,
            date_to=date_to,
            chamber_id_list=chamber_id_list,
            agent_id_list=agent_id_list,
            agent_deploy_id_list=agent_deploy_id_list,
            user_id_list=user_id_list,
            session_id_list=session_id_list,
            channel_id_list=channel_id_list,
            data_filter_list=data_filter_list,
            eval_id_user=eval_id_user,
            eval_id_internal=eval_id_internal,
            eval_id_auto=eval_id_auto,
            environment_id=environment_id,
            is_test_plan=is_test_plan,
            finished=finished,
            limit=limit,
            page_number=page_number,
            include_tags=include_tags,
            include_data=include_data,
            include_errors=include_errors,
            include_evals=include_evals
        )

        enola_result = self.__run_query()

        #show results
        return enola_result
    
    def __run_query(self) -> ExecutionModel:
        """
        Run Query, use self.execution_query_model
        """
        enola_result = get_execution(execution_query_model=self.execution_query_model, connection=self.connection, raise_error_if_fail=self.raise_error_if_fail)

        self.num_rows = len(enola_result.data)
        
        self.continue_execution = False
        if (self.num_rows != self.execution_query_model.limit and self.num_rows != 0):
            self.continue_execution = True

        self.num_rows_acum += self.num_rows

        return enola_result
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)
        



import jwt
from enola.base.common.huemul_functions import HuemulFunctions
from enola.base.internal.evaluation.enola_evaluation import create_evaluation
from enola.base.common.auth.auth_model import AuthModel
from enola.enola_types import EnolaSenderModel, EvaluationDetailModel, EvaluationModel, EvaluationResultModel, TokenInfo
from enola.base.connect import Connect


class Evaluation:
    def __init__(self, token, app_id=None, user_id=None, session_id=None, channel_id=None, ip=None, code_api=None, app_name:str="", user_name:str="", session_name:str="", channel_name:str=""):
        """
        Start Evaluation Execution

        token: jwt token, this is used to identify the agent, request from Admin App
        app_id: id of app, this is used to identify the app who is calling
        app_name: name of app, this is used to identify the app who is calling
        user_id: id of external user, this is used to identify the user who is calling
        user_name: name of external user, this is used to identify the user who is calling
        session_id: id of session of user or application, this is used to identify the session who is calling
        channel_id: web, chat, whatsapp, etc, this is used to identify the channel who is calling
        channel_name: web, chat, whatsapp, etc, this is used to identify the channel who is calling
        ip: ip of user or application, this is used to identify the ip who is calling
        code_api: code of api, this is used to identify the api who is calling
        """
        self.hf = HuemulFunctions()
        #Connection data

        #decodificar jwt
        self.token_info = TokenInfo(token=token)

        if (self.token_info.is_service_account == False):
            raise Exception("This token is not a service account. Only service accounts can execute evaluations.")

        if (self.token_info.is_service_account == True and self.token_info.service_account_can_evaluate == False):
            raise Exception("Service Account can't evaluate")


        self.connection = Connect(AuthModel(jwt_token=token, url_service=self.token_info.service_account_url, org_id=self.token_info.org_id))
        

        #user information
        self.enola_sender = EnolaSenderModel(app_id=app_id, app_name=app_name, user_id=user_id, user_name=user_name, session_id=session_id, session_name=session_name, channel_id=channel_id, channel_name=channel_name, ip=ip, code_api=code_api)

        #current date
        self.date_start = self.hf.get_date_for_api()
        self.executions = []
                
        #save steps and informations

    ########################################################################################
    ###############    E V A L U A T I O N     M E T H O D S     ###########################
    ########################################################################################


    def execution_exists(self, enola_id: str):
        """
        Check if execution exists
        enola_id: id of enola
        """
        for item in self.executions:
            if item.enola_id == enola_id:
                return item
        return None

    def add_evaluation(self, enola_id: str, eval_id: str, value: float, comment: str):
        """
        Add Evaluation
        enola_id: id of enola
        eval_id: id of evaluation
        value: value of evaluation
        comment: comment of evaluation
        """

        eval = EvaluationDetailModel(eval_id, value, comment)
        #check if enola_id exists (append eval), if not, create new enola_id
        execution = self.execution_exists(enola_id)


        if (execution is None):
            execution = EvaluationModel(enola_id, enola_sender = self.enola_sender)
            self.executions.append(execution)
        
        execution.add_eval(eval)

    def execute(self):
        """
        Execute Evaluations
        """
        final_result: EvaluationResultModel = EvaluationResultModel(
            total_evals=len(self.executions), 
            total_errors=0, 
            total_success=0, 
            errors=[]
        )
        
        for item in self.executions:
            result = create_evaluation(evaluation_model=item, connection=self.connection)
            if not result.successfull:
                print(result.errors)
                print(result.message)
                final_result.errors.append(result.message)

        return final_result
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)




from enola.base.connect import Connect
from enola.base.internal.tracking.enola_tracking_bloc import EnolaTrackingBloc
from enola.enola_types import TrackingModel, TrackingResponseModel


def create_tracking(tracking_model: TrackingModel, connection: Connect, raise_error_if_fail = True):
    #print("paso 20")
    if (not connection.can_execute):
        connection.huemul_logging.log_message_error(message = "cant execute: ")
        return TrackingResponseModel(
            enola_id="",
            agent_deploy_id="",
            url_evaluation_def_get="",
            url_evaluation_post="",
            successfull = False,
            message = "can't execute:"
        )

    #print("paso 30")
    connection.huemul_logging.log_message_info(message = "creating Enola Tracking")

    enola_tracking_result = EnolaTrackingBloc().enola_tracking_create(tracking_model=tracking_model,connect_object=connection)
    #print("paso 40")
    #if error
    if (not enola_tracking_result.isSuccessful):
        #print("paso 50")
        print(enola_tracking_result)
        connection._can_execute = False
        try:
            connection._error_message = enola_tracking_result.message if (len(enola_tracking_result.errors) == 0) else enola_tracking_result.errors[0]["errorTxt"]
        except:
            connection._error_message = enola_tracking_result.message if (len(enola_tracking_result.errors) == 0) else enola_tracking_result.errors[0].errorTxt

        #print("paso 60")

        connection.huemul_logging.log_message_error(message = "error in enolaTracking: " + connection._error_message)

        if (raise_error_if_fail):
            connection.huemul_logging.log_message_error(message = "error" + connection._error_message)
            raise NameError(connection._error_message)
        else:
            connection.huemul_logging.log_message_error(message = "error" + connection._error_message)
            return TrackingResponseModel(
                enola_id="",
                agent_deploy_id="",
                url_evaluation_def_get="",
                url_evaluation_post="",
                successfull = False,
                message = "error" + connection._error_message
            )

    #if all ok, continue
    # connectObject._processExecStepId = enolaAgentResult.data[0].processExecStepId

    return TrackingResponseModel(
        enola_id=enola_tracking_result.data.get("enolaId"),
        agent_deploy_id=enola_tracking_result.data.get("agentDeployId"),
        url_evaluation_def_get=enola_tracking_result.data.get("urlEvaluationDefGet"),
        url_evaluation_post=enola_tracking_result.data.get("urlEvaluationPost"),
        successfull = enola_tracking_result.isSuccessful,
        message = enola_tracking_result.message,
        **enola_tracking_result.data
    )


from datetime import datetime, timezone

class HuemulFunctions:
    def __init__(self):
        self.started = True
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)
    
    #
    # getCurrentDateTime: returns current datetime
    # from version 1.1
    #
    def get_current_datetime_java(self):
        return datetime.now(timezone.utc)

    # return datetime in log format
    # date: datetime.now(timezone.utc)
    # return string
    def get_date_for_log(self, date):
        #dateTimeFormat: DateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS")
        dateString = date.isoformat(timespec='milliseconds')

        return dateString

    # return date in string format
    # date
    def get_date_for_api(self):
        #dateTimeFormat: DateFormat = new SimpleDateFormat("yyyy-MM-ddTHH:mm:ss:SSSz")
        newDate = datetime.now(timezone.utc)
        dateString = newDate.isoformat(timespec='milliseconds')

        return dateString
    
    def get_dif_seconds(self, dateFromIso, dateToIso):
        dateFrom = datetime.fromisoformat(dateFromIso)
        dateTo = datetime.fromisoformat(dateToIso)
        return (dateTo - dateFrom).total_seconds()
    
    def get_dif_ms(self, dateFromIso, dateToIso):
        dateFrom = datetime.fromisoformat(dateFromIso)
        dateTo = datetime.fromisoformat(dateToIso)
        return (dateTo - dateFrom).total_seconds() * 1000

    def delete_args(self, data):
        try:
            delattr(data, "args")
        except:
            a = None


import logging

class HuemulLogging:
    def __init__(self):
        FORMAT = '%(asctime)s %(message)s'
        logging.basicConfig(format=FORMAT, level=logging.INFO)
        self.logger = logging.getLogger('Enola')

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

    #
    # logMessageDebug: Send {message} to log4j - Debug
    #
    def logMessageDebug(self, message):
        self.logger.debug(message)

    #
    # logMessageInfo: Send {message} to log4j - Info
    #
    def log_message_info(self, message):
        self.logger.info(message)

    #
    # logMessageWarn: Send {message} to log4j - Warning
    #
    def logMessageWarn(self, message):
        self.logger.warn(message)

    #
    # logMessageError: Send {message} to log4j - Error
    #
    def log_message_error(self, message):
        self.logger.error(msg = str(message), extra={"error": message})


import json
from enola.base.common.huemul_functions import HuemulFunctions
from enola.base.common.huemul_connection import HuemulConnection
from enola.base.common.huemul_response_error import HuemulResponseError
from enola.base.common.huemul_response_to_bloc import HuemulResponseToBloc
from enola.enola_types import EvaluationModel, EvaluationResponseModel

class EnolaEvaluationProvider(HuemulResponseToBloc):
    
    #
    # evaluation_create
    # @param EvaluationModel evaluationModel
    # @return AgentExecuteResponseModel[AgentExecuteResponseModel]
    #
    def evaluation_create(self, evaluation_model: EvaluationModel):
        #self = AgentExecuteResponseModel()
        try:
            hf = HuemulFunctions()
            hf.delete_args(evaluation_model)
            #dataIn2 = jsonpickle.encode(agentModel)
            data_in = json.dumps(evaluation_model.to_json())

            self.message = "starting postRequest"
            huemul_response = HuemulConnection(connect_object=self.connect_object).post_request(
                route = "agent/eval/v1/",
                data = data_in,
            )

            #get status from connection
            self.message = "starting fromResponseProvider"
            self.from_response_provider(huemul_response_provider = huemul_response)
            if (self.isSuccessful):
                self.data = EvaluationResponseModel(**huemul_response.data_raw)
        except Exception as e:
            self.errors.append(
                HuemulResponseError(errorId = "APP-101", errorTxt = str(e))
            )

        return self


from enola.base.connect import Connect
from enola.base.internal.evaluation.enola_evaluation_bloc import EnolaEvaluationBloc
from enola.enola_types import EvaluationModel, EvaluationResponseModel


def create_evaluation(evaluation_model: EvaluationModel, connection: Connect, raise_error_if_fail = True):
    if (not connection.can_execute):
        connection.huemul_logging.log_message_error(message = "can't execute: ")
        return EvaluationResponseModel(
            enola_id = "",
            agent_deploy_id = "",
            enola_eval_id = "",
            successfull = False,
            message = "can't execute:"
        )

    connection.huemul_logging.log_message_info(message = "creating Enola Evaluation")

    enola_evaluation_result = EnolaEvaluationBloc().enola_evaluation_create(evaluation_model=evaluation_model,connect_object=connection)
    #if error
    if (not enola_evaluation_result.isSuccessful):
        print(enola_evaluation_result)
        connection._can_execute = False
        try:
            connection._error_message = enola_evaluation_result.message if (len(enola_evaluation_result.errors) == 0) else enola_evaluation_result.errors[0]["errorTxt"]
        except:
            connection._error_message = enola_evaluation_result.message if (len(enola_evaluation_result.errors) == 0) else enola_evaluation_result.errors[0].errorTxt

        connection.huemul_logging.log_message_error(message = "error in enolaEvaluation: " + connection._error_message)

        if (raise_error_if_fail):
            connection.huemul_logging.log_message_error(message = "error" + connection._error_message)
            raise NameError(connection._error_message)
        else:
            connection.huemul_logging.log_message_error(message = "error" + connection._error_message)
            return EvaluationResponseModel(
                enola_id = "",
                agent_deploy_id = "",
                enola_eval_id = "",
                successfull = False,
                message = "error" + connection._error_message
            )

    #if all ok, continue
    # connectObject._processExecStepId = enolaAgentResult.data[0].processExecStepId

    return EvaluationResponseModel(
        successfull = enola_evaluation_result.isSuccessful,
        message = enola_evaluation_result.message,
        enola_eval_id= enola_evaluation_result.data.enola_eval_id,
        enola_id = enola_evaluation_result.data.enola_id,
        agent_deploy_id = enola_evaluation_result.data.agent_deploy_id
    )


from enola.base.connect import Connect
from enola.base.internal.tracking.enola_tracking_provider import EnolaTrackingProvider
from enola.enola_types import TrackingModel


class EnolaTrackingBloc():
    #
    # start enolaAgentCreate
    # @param AgentModel AgentModel
    # @return HuemulResponseBloc[EnolaAgentResponseModel]
    #
    def enola_tracking_create(self, tracking_model: TrackingModel, connect_object: Connect):
        (continue_in_loop) = True
        attempt = 0
        #result = HuemulResponseToBloc(connectObject=connectObject)

        while ((continue_in_loop)):
            result = EnolaTrackingProvider(connect_object=connect_object).tracking_create(
                    tracking_model=tracking_model
            )
            attempt +=1
            (continue_in_loop) = result.analyze_errors(attempt)
        
        return result


#
# @author Sebastián Rodríguez Robotham
# receive some params
# @param name param name
# @param value param value
#
class HuemulParam:
    def __init__(self, name, value):
        self.name = name
        self.value = value

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)



# consumer_id: String,
# consumer_secret: String,
# org_id: String,
# application_name: String,
# url_service: String
class AuthModel:
    def __init__(self, consumer_id:str = "", consumer_secret:str = "", org_id:str = "", application_name:str = "", url_service:str = "", session_id:str = "", jwt_token: str = ""):
        self.consumer_id = consumer_id
        self.consumer_secret = consumer_secret
        self.org_id = org_id
        self.application_name = application_name
        self.url_service = url_service
        self.sessionId = session_id
        self.jwt_token = jwt_token
        


from enola import get_executions
from enola.enola_types import CompareType, DataType, ExecutionDataFilter
#from enola.enola_types import ErrOrWarnKind
#from enola.base.enola_types import AgentResponseModel

token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiJFTk9MQV9IVUVNVUwwNC1mYTIyOTZkNDBiYjUwMzNkYTdhZjE1N2JiMzUwYjc2ZiIsImlkIjoiZmNhZDY2MjYtMjM3MS00NTMxLWIwMDItNzhhODFhNjk5ZTdjIiwiZGlzcGxheU5hbWUiOiJudWV2aXRhIDEgLSBzb2xvIGdldCIsImFnZW50RGVwbG95SWQiOiJFTk9MQV9IVUVNVUwwNC1mYTIyOTZkNDBiYjUwMzNkYTdhZjE1N2JiMzUwYjc2ZiIsImNhblRyYWNraW5nIjpmYWxzZSwiY2FuRXZhbHVhdGUiOmZhbHNlLCJjYW5HZXRFeGVjdXRpb25zIjp0cnVlLCJ1cmwiOiJodHRwOi8vbG9jYWxob3N0OjcwNzIvYXBpIiwidXJsQmFja2VuZCI6Imh0dHA6Ly9sb2NhbGhvc3Q6NzA3MS9hcGkiLCJvcmdJZCI6IkVOT0xBX0hVRU1VTDA0IiwiaXNTZXJ2aWNlQWNjb3VudCI6dHJ1ZSwiaWF0IjoxNzE5MTYzNTA5LCJleHAiOjE4NDUyNTkxOTksImlzcyI6ImVub2xhIn0.wxcrvVB5lDtzm9VkYtilGXXWZIQsR820SQdS4ymkHUM"
exec = get_executions.GetExecutions(token=token, raise_error_if_fail=False)

a = exec.get_items(
    date_from="2024-06-23t22:00", 
    date_to="2024-06-25", 
    #eval_id_auto=ExecutionEvalFilter(eval_id=["0", "11"], include=False),
    data_filter_list=[
        #ExecutionDataFilter(name="Input usuario", value="value1", type=DataType.STRING, compare=CompareType.CONTAINS),
        #ExecutionDataFilter(name="Input usuario", value="1894992", compare=CompareType.CONTAINS),
        #ExecutionDataFilter(name="info3", value="10", compare=CompareType.LESS_EQUAL, type=DataType.NUMBER),
    ],
    #agent_deploy_id=["ENOLA_HUEMUL04-fa2296d40bb5033da7af157bb350b76f"]
    #agent_deploy_id=["ENOLA_HUEMUL04-b11664d1b96d1df3f7e395ae3374d273"]
    )

print(len(a.data))
print(a.message)







from enola.base.connect import Connect
from enola.base.internal.executions.enola_execution_provider import EnolaExecutionProvider
from enola.enola_types import ExecutionQueryModel

class EnolaExecutionBloc():
    #
    # start enolaExecutionGet
    # @param AgentModel AgentModel
    # @return HuemulResponseBloc[EnolaAgentResponseModel]
    #
    def enola_execution_get(self, execution_query_model: ExecutionQueryModel, connect_object: Connect):
        (continue_in_loop) = True
        attempt = 0
        #result = HuemulResponseToBloc(connectObject=connectObject)

        while ((continue_in_loop)):
            result = EnolaExecutionProvider(connect_object=connect_object).execution_get(
                    execution_query_model=execution_query_model
            )
            attempt +=1
            (continue_in_loop) = result.analyze_errors(attempt)
        
        return result


 # errorTrace, 
 # errorClassName, 
 # errorFileName, 
 # errorLineNumber, 
 # errorMethodName, 
 # errorMessage, 
 # errorIsError, 
 # errorCode
class HuemulError:
    def __init__(self, org_id, huemul_logging, **args):
        self.errorId = ""
        self.org_id = org_id
        self.error_detail = ""
        self.error_who_is = ""
        self.error_who_code = ""
        self.huemul_logging = huemul_logging

        if (len(args) == 0):
            #val Invoker: Array[StackTraceElement] = new Exception().getStackTrace
            self.error_is_error = False
            self.error_code = ""
            self.error_trace = ""
            self.error_class_name = ""
            self.error_file_name = "" # Invoker(3).getFileName //todo: obtener dinamicamente a partir de los nombres de las clases (descartando)
            self.error_line_number = "" #Invoker(3).getLineNumber
            self.error_method_name = ""
            self.error_message = ""
        elif (len(args) == 4):
            #val Invoker: Array[StackTraceElement] = new Exception().getStackTrace
            self.error_is_error = True
            self.error_code = args["code"] if args["code"] != None else args["error_code"]
            self.error_trace = ""
            self.error_class_name = args["getClassName"] if args["getClassName"] != None else args["error_class_name"]
            self.error_file_name = "" # Invoker(3).getFileName //todo: obtener dinamicamente a partir de los nombres de las clases (descartando)
            self.error_line_number = "" #Invoker(3).getLineNumber
            self.error_method_name = args["getMethodName"] if args["getMethodName"] != None else args["error_method_name"]
            self.error_message = args["message"] if args["message"] != None else args["error_message"]

            self.printError(self.error_message)
        elif len(args) > 4:
            self.error_trace = args["errorTrace"] if args["errorTrace"] != None else args["error_trace"]
            self.error_class_name = args["errorClassName"] if args["errorClassName"] != None else args["error_class_name"]
            self.error_file_name = args["errorFileName"] if args["errorFileName"] != None else args["error_file_name"]
            self.error_line_number = args["errorLineNumber"] if args["errorLineNumber"] != None else args["error_line_number"]
            self.error_method_name = args["errorMethodName"] if args["errorMethodName"] != None else args["error_method_name"]
            self.error_message = args["errorMessage"] if args["errorMessage"] != None else args["error_message"]
            self.error_is_error = args["errorIsError"] if args["errorIsError"] != None else args["error_is_error"]
            self.error_code = args["errorCode"] if args["errorCode"] != None else args["error_code"]

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

    # return boolean
    def isOK(self):
        return not (self.error_is_error)

    
    def printError(self, error):
        self.huemul_logging.logMessageError("***************************************************************")
        self.huemul_logging.logMessageError("HuemulLauncher: Error Detail")
        self.huemul_logging.logMessageError("***************************************************************")
        self.huemul_logging.logMessageError("error_ClassName: " + self.error_class_name)
        self.huemul_logging.logMessageError("error_FileName: " + self.error_file_name)
        self.huemul_logging.logMessageError("error_ErrorCode: " + self.error_code)
        self.huemul_logging.logMessageError("error_LineNumber: " + self.error_line_number)
        self.huemul_logging.logMessageError("error_MethodName: " + self.error_method_name)
        self.huemul_logging.logMessageError("error_Message: " + self.error_message)
        self.huemul_logging.logMessageError("error_Trace: " + self.error_trace)

        self.huemul_logging.logMessageError("Detail")
        self.huemul_logging.logMessageError(error)




#
# @author Sebastián Rodríguez Robotham
# Return datetime details
# @param difDates endDateTime - StartDateTime (long)
#
class HuemulDateTimePart:
    def __init__(self, difDates):
        milliseconds = difDates
        calc = difDates / 1000
        self.second = calc % 60
        calc /= 60
        self.minute = calc % 60
        calc /= 60
        self.hour = calc % 24
        calc /= 24
        self.days = calc

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)


#
# @author Sebastián Rodríguez Robotham
# error class used by backend to return error info
# @param errorId error Id
# @param errorTxt error Message
#
class HuemulResponseError:
    def __init__(self, errorId, errorTxt):
        self.errorId = errorId
        self.errorTxt = errorTxt


    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)


from enum import Enum
import json

import jwt



class Environtment(Enum):
    DEV = "DEV"
    QA = "QA"
    PROD = "PROD"

class DataType(Enum):
    TEXT = "TEXT"
    NUMBER = "NUMBER"
    DATE = "DATE"
    BOOLEAN = "BOOLEAN"

class CompareType(Enum):
    EQUAL = "EQUAL"
    GREATER = "GREATER"
    LESS = "LESS"
    GREATER_EQUAL = "GREATER_EQUAL"
    LESS_EQUAL = "LESS_EQUAL"
    NOT_EQUAL = "NOT_EQUAL"
    CONTAINS = "CONTAINS"

class TokenInfo:
    def __init__(self, token: str):

        if token == "":
                raise Exception("token is empty.")
        
        try:
            decoded = jwt.decode(token, algorithms=['none'], options={'verify_signature': False})
            self.agent_deploy_id = decoded.get("agentDeployId", None)
            self.org_id = decoded.get("orgId", None)
            self.service_account_id = decoded.get("id", None)
            self.service_account_name = decoded.get("displayName", None)
            self.service_account_url = decoded.get("url", None)
            self.service_account_url_backend = decoded.get("urlBackend", None)
            self.service_account_can_tracking = decoded.get("canTracking", False)
            self.service_account_can_evaluate = decoded.get("canEvaluate", False)
            self.is_service_account = decoded.get("isServiceAccount", False)
            self.service_account_can_get_executions = decoded.get("canGetExecutions", False)

            #verify if serviceAccountUrl is empty, return error
            if not self.service_account_url:
                raise Exception("serviceAccountUrl is empty.")
            if not self.service_account_url_backend:
                raise Exception("serviceAccountUrlBackend is empty.")
            if not self.org_id:
                raise Exception("orgId is empty.")
            
        except jwt.ExpiredSignatureError:
            print("token expired.")
        except jwt.DecodeError:
            print("Error decoding token.")
        except jwt.InvalidTokenError:
            print("Invalid Token.")


    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)
        

class KindType(Enum):
    RECEIVER = "RECEIVER"
    SENDER = "SENDER"

class ErrorType(Enum):
    ERROR = "ERROR"
    WARNING = "WARNING"


class ErrOrWarnKind(Enum):
    """
    EXTERNAL: external agent call generate an unexpected error or warning
    """
    EXTERNAL = "EXTERNAL"
    """
    INTERNAL_CONTROLLED: internal agent call generate an unexpected error or warning
    """
    INTERNAL_CONTROLLED = "INTERNAL_CONTROLLED"
    """
    INTERNAL_TOUSER: controlled error or warning to send to user
    """
    INTERNAL_TOUSER = "INTERNAL_TOUSER"

class ErrorOrWarnModel:
    def __init__(self, id: str, message: str, error_type: ErrorType, kind: ErrOrWarnKind):
        self.id = id
        self.error = message
        self.error_type: ErrorType = error_type
        self.kind: ErrOrWarnKind = kind

    def to_json(self):
        return {
            "id": self.id,
            "error": self.error,
            "error_type": self.error_type.value,
            "kind": self.kind.value
        }
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)
    
class DataListModel:
    def __init__(self, kind: KindType, name: str, data_type: DataType, value):
        self.kind = kind
        self.name = name
        self.data_type = data_type
        self.value = value

    def to_json(self):
        return {
            "kind": self.kind.value,
            "name": self.name,
            "data_type": self.data_type.value,
            "value": self.value
        }
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class Info:

    def is_numeric(self, value):
        return isinstance(value, (int, float, complex))
    def is_string(self, value):
        return isinstance(value, (str))
    def is_dict(self, value):
        return isinstance(value, (dict))

    def __init__(self, type: str, key: str, value):
        self.type = type
        self.key = key
        #si valor es numerico, asignar
        if (self.is_numeric(value)):
            self.value = value
        elif (self.is_string(value)):
            if (self.is_dict(value)):
                self.value = json.dumps(value)
            else:
                self.value = value
        else:
            self.value = value
        

    def to_json(self):
        return {
            "type": self.type,
            "key": self.key,
            "value": self.value
        }
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class ApiDataModel:
    def __init__(self, name: str, method: str, url: str, body: str, header: str, payload: str, description: str):
        self.name = name
        self.method = method
        self.url = url
        self.description = description
        self.body = body
        self.header = header
        self.payload = payload


    def to_json(self):
        return {
            "name": self.name,
            "method": self.method,
            "url": self.url,
            "description": self.description,
            "body": self.body,
            "header": self.header,
            "payload": self.payload
        }
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class FileInfoModel:
    def __init__(self, name: str, url: str, type: str, sizeKb: int, description: str):
        self.name = name
        self.url = url
        self.type = type
        self.size = sizeKb
        self.description = description

    def to_json(self):
        return {
            "name": self.name,
            "url": self.url,
            "type": self.type,
            "size": self.size,
            "description": self.description
        }
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)
    
class EnolaSenderModel:
    """
    EnolaSenderModel
    """
    def __init__ (self, app_id: str, app_name: str, user_id: str, user_name: str, session_id: str, session_name: str, channel_id: str, channel_name: str, ip: str, code_api: str):
        self.app_id=app_id
        self.app_name=app_name
        self.user_id=user_id
        self.user_name=user_name
        self.session_id=session_id
        self.session_name=session_name
        self.channel_id=channel_id
        self.channel_name=channel_name
        self.ip=ip
        self.code_api=code_api

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

#***********************************************************************************
#*************   T R A C K I N G   T Y P E S     ***********************************
#***********************************************************************************

class TrackingModel:
    """
    TrackingModel
    """
    def __init__(self, isTest: bool, step_list: list, steps: int, enola_id_prev: str, enola_sender: EnolaSenderModel):
        self.enola_sender = enola_sender
        
        self.isTest = isTest
        #step_list es una lista, generar un arreglo ejecutando el metodo to_json de cada elemento de la lista con map
        self.step_list = step_list
        self.steps = steps
        self.enola_id_prev = enola_id_prev

    def to_json(self):
        return {
            "app_id": self.enola_sender.app_id,
            "app_name": self.enola_sender.app_name,
            "user_id": self.enola_sender.user_id,
            "user_name": self.enola_sender.user_name,
            "session_id": self.enola_sender.session_id,
            "channel_id": self.enola_sender.channel_id,
            "session_name": self.enola_sender.session_name,
            "channel_name": self.enola_sender.channel_name,
            "ip": self.enola_sender.ip,
            "code_api": self.enola_sender.code_api,
            "isTest": self.isTest,
            "step_list": list(map(lambda x: x.to_json(), self.step_list)),
            "steps": self.steps,
            "enola_id_prev": self.enola_id_prev
        }
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class TrackingResponseModel:
    def __init__(self, enola_id: str, agent_deploy_id:str, successfull: bool, message: str, url_evaluation_def_get: str="", url_evaluation_post: str="",  **args):
        self.enola_id = enola_id
        self.agent_deploy_id = agent_deploy_id
        self.url_evaluation_def_get = url_evaluation_def_get
        self.url_evaluation_post = url_evaluation_post
        self.successfull = successfull
        self.message = message
        self.args = args

    def to_json(self):
        return {
            "enolaId": self.enola_id,
            "agentDeployId": self.agent_deploy_id,
            "successfull": self.successfull,
            "message": self.message,
            "args": self.args
        }
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)


#***********************************************************************************
#*************   E X E C U T I O N   T Y P E S     ***********************************
#***********************************************************************************

class ExecutionModel:
    def __init__(self, data: list, successfull: bool, message: str, **args):
        self.data = data
        self.successfull = successfull
        self.message = message
        self.args = args

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class ExecutionEvalFilter:
    def __init__(self, eval_id: list, include: bool = True):
        self.eval_id = eval_id
        self.include = include

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class ExecutionDataFilter:
    def __init__(self, name: str, value, type: DataType = DataType.TEXT, compare: CompareType = CompareType.EQUAL):
        self.name = name
        self.value = value
        self.type = type
        self.compare = compare

    def to_json(self):
        return {
            "name": self.name,
            "value": self.value,
            "type": self.type.value,
            "compare": self.compare.value
        }
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)
    

class ExecutionQueryModel:
    def __init__(self,
        date_from:str,
        date_to:str,
        chamber_id_list:list = [], 
        agent_id_list:list = [], 
        agent_deploy_id_list:list = [],
        user_id_list:list = [],
        session_id_list:list = [],
        channel_id_list:list = [],
        data_filter_list:list = [], #ExecutionDataFilter
        eval_id_user: ExecutionEvalFilter = None,
        eval_id_internal: ExecutionEvalFilter = None,
        eval_id_auto: ExecutionEvalFilter = None,
        environment_id:Environtment = None,
        is_test_plan: bool= None,
        finished:bool = None,
        limit:int=100, 
        page_number:int=1, 
        include_tags:bool=False,
        include_data:bool=False,
        include_errors:bool=False,
        include_evals:bool=False):
        self.date_from = date_from
        self.date_to = date_to
        self.chamber_id_list = chamber_id_list
        self.agent_id_list = agent_id_list
        self.agent_deploy_id_list = agent_deploy_id_list
        self.user_id_list = user_id_list
        self.session_id_list = session_id_list
        self.channel_id_list = channel_id_list
        self.data_filter_list = data_filter_list
        self.eval_id_user = eval_id_user
        self.eval_id_internal = eval_id_internal
        self.eval_id_auto = eval_id_auto
        self.environment_id = environment_id
        self.isTestPlan = is_test_plan
        self.finished = finished
        self.limit = limit
        self.page_number = page_number
        self.includeTags = include_tags
        self.includeData = include_data
        self.includeErrors = include_errors
        self.includeEvals = include_evals

        if (date_from == ""):
            raise Exception("date_from is empty.")
        if (date_to == ""):
            raise Exception("date_to is empty.")
        if (limit == 0):
            raise Exception("limit is 0.")
        if (page_number == 0):
            raise Exception("page is 0.")
        if (limit < 1):
            raise Exception("limit must be greater than 0.")
        if (page_number < 1):
            raise Exception("page_number must be greater than 0.")
        
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)
        
class ExecutionResponseModel:
    def __init__(self,
                agentExecId: str,
                agentExecIdRelated: str,
                agentDeployId: str,
                agentDeployName: str,
                agentId: str,
                agentName: str,
                agentExecName: str,
                agentExecStartDT: str,
                agentExecEndDT: str,
                agentExecDurationMs: int,
                agentExecNumTracking: str,
                agentExecIsTest: bool,
                environmentId: str,

                agentExecCliAppId: str,
                agentExecCliAppName: str,
                agentExecCliUserId: str,
                agentExecCliUserName: str,
                agentExecCliSessionId: str,
                agentExecCliSessionName: str,
                agentExecCliChannel: str,
                agentExecCliChannelName: str,
                agentExecMessageInput : str,
                agentExecMessageOutput : str,
                agentExecTagJson : json,
                agentExecFileInfoJson : json,
                agentExecDataJson : json,
                agentExecErrorOrWarningJson : json,
                agentExecStepApiDataJson : json,
                agentExecInfoJson : json,
                agentExecEvals: json,
                agentExecCliIP: str,
                agentExecCliNumIter: int,
                agentExecCliCodeApi: str,
                agentExecSuccessfull: bool,
                **args
                 ):
        self.enola_id = agentExecId
        self.enola_id_related = agentExecIdRelated
        self.agent_deploy_id = agentDeployId
        self.agent_deploy_name = agentDeployName
        self.agent_id = agentId
        self.agent_name = agentName
        self.name = agentExecName
        self.start_dt = agentExecStartDT
        self.end_dt = agentExecEndDT
        self.duration_ms = agentExecDurationMs
        self.num_tracking = agentExecNumTracking
        self.is_test = agentExecIsTest
        self.environment_id = environmentId
        self.app_id = agentExecCliAppId
        self.app_name = agentExecCliAppName
        self.user_id = agentExecCliUserId
        self.user_name = agentExecCliUserName
        self.session_id = agentExecCliSessionId
        self.session_name = agentExecCliSessionName
        self.channel = agentExecCliChannel
        self.channel_name = agentExecCliChannelName
        self.message_input = agentExecMessageInput
        self.message_output = agentExecMessageOutput
        self.tag_json = agentExecTagJson
        self.file_info_json = agentExecFileInfoJson
        self.data_json = agentExecDataJson
        self.error_or_warning_json = agentExecErrorOrWarningJson
        self.step_api_data_json = agentExecStepApiDataJson
        self.info_json = agentExecInfoJson
        self.evals = agentExecEvals
        self.ip = agentExecCliIP
        self.num_iter = agentExecCliNumIter
        self.code_api = agentExecCliCodeApi
        self.successfull = agentExecSuccessfull

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

#***********************************************************************************
#*************   E V A L U A T I O N   T Y P E S     ***********************************
#***********************************************************************************

class EvaluationResultModel:
    def __init__(self, total_evals: int, total_errors: int, total_success: int, errors: list):
        self.total_evals = total_evals
        self.total_errors = total_errors
        self.total_success = total_success
        self.errors = errors

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class EvaluationDetailModel:
    """
    EvaluationDetailModel
    """
    def __init__(self, eval_id: str, value: float, comment: str):
        self.eval_id = eval_id
        self.value = value
        self.comment = comment

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class EvaluationModel:
    """
    EvaluationModel
    """
    def __init__(self, enola_id: str, enola_sender: EnolaSenderModel):
        self.enola_id = enola_id
        self.evals = []
        self.enola_sender = enola_sender

    def add_eval(self, eval: EvaluationDetailModel):
        self.evals.append(eval)

    def to_json(self):
        result = {
            "enolaId": self.enola_id,
            "evalType": "AUTO",
            "sender": {
                "app_id": self.enola_sender.app_id,
                "app_name": self.enola_sender.app_name,
                "user_id": self.enola_sender.user_id,
                "user_name": self.enola_sender.user_name,
                "session_id": self.enola_sender.session_id,
                "session_name": self.enola_sender.session_name,
                "channel_id": self.enola_sender.channel_id,
                "channel_name": self.enola_sender.channel_name,
                "ip": self.enola_sender.ip,
                "code_api": self.enola_sender.code_api
            },
            "evals": {
                item.eval_id: {
                    "value": item.value,
                    "comment": item.comment
                }
                for item in self.evals
            }
        }
        return result;

    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

class EvaluationResponseModel:
    def __init__(self, enola_id: str="", agent_deploy_id:str="", enola_eval_id: str="", successfull: bool=True, message: str = "",  **args):
        self.enola_id = enola_id if enola_id != "" else args.get("enolaId", None)
        self.agent_deploy_id = agent_deploy_id if agent_deploy_id != "" else args.get("agentDeployId", None)
        self.enola_eval_id = enola_eval_id if enola_eval_id != "" else args.get("enolaEvalId", None)
        self.successfull = successfull if successfull != "" else args.get("IsSuccessfull", None)
        self.message = message
        self.args = args

    def to_json(self):
        return {
            "enolaId": self.enola_id,
            "agentDeployId": self.agent_deploy_id,
            "enolaEvalId": self.enola_eval_id,
            "successfull": self.successfull,
            "message": self.message,
            "args": self.args
        }
    
    def __getitem__(self, key):
        return self.__dict__[key]

    def __getattr__(self, key):
        return self.__dict__[key]

    def get(self, key, default=None):
        return self.__dict__.get(key, default)




from enola.base.common.auth.auth_service_provider import AuthServiceProvider

class AuthServiceBloc():
    #
    # start authSignInService
    # @param authModel authModel
    # @return HuemulResponseBloc[AuthServiceModel]
    #
    def authSignInService(self, authModel, connectObject):
        continueInLoop = True
        attempt = 0
        #result = HuemulResponseBloc()

        connectObject.huemulLogging.logMessageInfo("Ground Control station: " + authModel.urlService)
        connectObject.huemulCommon.setServiceUrl(value = authModel.urlService)

        while (continueInLoop):
            result = AuthServiceProvider(connect_object=connectObject).authSignInService(
                consumer_id = authModel.consumer_id,
                consumer_secret = authModel.consumer_secret,
                org_id = authModel.org_id,
                application_name = authModel.application_name
            )

            attempt +=1
            continueInLoop = result.analyze_errors(attempt)
        

        return result


from enola import evaluation
#from enola.enola_types import ErrOrWarnKind
#from enola.base.enola_types import AgentResponseModel

token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiJFTk9MQV9IVUVNVUwwNC1mYTIyOTZkNDBiYjUwMzNkYTdhZjE1N2JiMzUwYjc2ZiIsImlkIjoiOWVjMTVlZDEtYTkzNy00ZTRlLTkwZTQtZmQ3YmNiMGRmOTg0IiwiZGlzcGxheU5hbWUiOiJudWV2aXRhIDEgLSBzb2xvIGV2YWx1YSIsImFnZW50RGVwbG95SWQiOiJFTk9MQV9IVUVNVUwwNC1mYTIyOTZkNDBiYjUwMzNkYTdhZjE1N2JiMzUwYjc2ZiIsImNhblRyYWNraW5nIjpmYWxzZSwiY2FuRXZhbHVhdGUiOnRydWUsImNhbkdldEV4ZWN1dGlvbnMiOmZhbHNlLCJ1cmwiOiJodHRwOi8vbG9jYWxob3N0OjcwNzIvYXBpIiwidXJsQmFja2VuZCI6Imh0dHA6Ly9sb2NhbGhvc3Q6NzA3MS9hcGkiLCJvcmdJZCI6IkVOT0xBX0hVRU1VTDA0IiwiaXNTZXJ2aWNlQWNjb3VudCI6dHJ1ZSwiaWF0IjoxNzE5MTYzNDY3LCJleHAiOjE4NDUyNTkyMDgsImlzcyI6ImVub2xhIn0.tIQvvOzmrcfDDKyen4d1Djxfvr-9XvMlR6wLl6l_8ag"
eval = evaluation.Evaluation(token=token)

eval.add_evaluation(enola_id="8c28bacd8b43bb74c0e070fb5145105c61c9623c4aef8fc5932af963f3d17171", eval_id="0", value=100, comment="eval 0, valor 1, AUTO")
eval.add_evaluation(enola_id="8c28bacd8b43bb74c0e070fb5145105c61c9623c4aef8fc5932af963f3d17171", eval_id="11", value=50, comment="eval 10, valor 2, AUTO")
#eval.add_evaluation(enola_id="7875bc29361e7ceff45258461fd16e4db4638c2fc73cb33e887dc40fff0047a92", eval_id="2'", value=3, comment="eval 2, valor 3")


result = eval.execute()

print ("total evals: ", result.total_evals)
print ("total errors: ", result.total_errors)


